/*
 * Copyright 2010-2019 Max Kellermann <max.kellermann@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef JAVA_OBJECT_HXX
#define JAVA_OBJECT_HXX

#include "Ref.hxx"

#include <jni.h>

#include <assert.h>

namespace Java {
	/**
	 * Wrapper for a local "jobject" reference.
	 */
	typedef LocalRef<jobject> LocalObject;

	class GlobalObject : public GlobalRef<jobject> {
	public:
		/**
		 * Constructs an uninitialized object.  The method
		 * set() must be called before it is destructed.
		 */
		GlobalObject() = default;

		GlobalObject(JNIEnv *env, jobject obj) noexcept
			:GlobalRef<jobject>(env, obj) {}
	};

	/**
	 * Utilities for java.net.Object.
	 */
	class Object {
		static jmethodID toString_method;

	public:
		static void Initialise(JNIEnv *env);

		static jstring toString(JNIEnv *env, jobject o) {
			assert(env != nullptr);
			assert(o != nullptr);
			assert(toString_method != nullptr);

			return (jstring)env->CallObjectMethod(o, toString_method);
		}
	};
}

#endif
